// CALIBRATION (3-AXIS ACCELEROMETER ADXL326)
// Author : Solène Kojtych
// solene.kojtych@polymtl.ca
// 2018


//Wiring (accelerometer pin -------- Arduino pin)
// power                            :     Vin     --------   power 3.3V 
// ground                          :     GND   --------   GND
// voltage reference               :     3Vo     --------   AREF
// X axis                              :      Xout  --------   A2
// Y axis                              :      Yout  --------   A1
// Z axis                              :       Zout --------   A0

// ------------------ Parameters
// Connexion pins
const int xPin = A2;                  // X axis
const int yPin = A1;                  // Y axis
const int zPin = A0;                  // Z axis

// Constants 
   // Nb of measures for each position
const int samplingsNb = 10;                   
  //  Data rate in bits/s for serial data transmission                    
const unsigned int serialDataRate = 57600;   
 
// ---------------- initialization
// Variables
int xPlus = 0;
int xMinus = 0;

int yPlus = 0;
int yMinus = 0;

int zPlus = 0;
int zMinus = 0;

int xZero = 0;
int yZero = 0;
int zZero = 0;

char pythonMessage ='0';

void setup() 
{// sets voltage reference for  the analog-to-digital converter
  analogReference(EXTERNAL);  
// starts serial communication with computer 
  Serial.begin(serialDataRate);    
}

void loop() 
{// Wait for start trigger
Serial.println("*Appuyer sur 1 pour débuter la calibration");  
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
pythonMessage ='0';

//------------ Procedure de calibration ---------
  // Z+ axis 
Serial.println("*Positionner l'axe Z+ vers le plafond et appuyer sur 1");    
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      // Samples reading
pythonMessage ='0';
xZero = ReadAxis(xPin);
yZero = ReadAxis(yPin);
zPlus = ReadAxis(zPin);

  // Z- axis
Serial.println("*Positionner l'axe Z- vers le plafond et appuyer sur 1");  
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      //  Samples reading
pythonMessage ='0';
zMinus = ReadAxis(zPin);

  // Axe X +
Serial.println("*Positionner l'axe X+ vers le plafond et appuyer sur 1");    
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      // Samples reading
pythonMessage ='0';
xPlus = ReadAxis(xPin);
zZero = ReadAxis(zPin);

  // X- axis
Serial.println("*Positionner l'axe X- vers le plafond et appuyer sur 1");    
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      // Samples reading
pythonMessage ='0';
xMinus = ReadAxis(xPin);

  // Y+ axis
Serial.println("*Positionner l'axe Y+ vers le plafond et appuyer sur 1");    
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      // Samples reading
pythonMessage ='0';
yPlus = ReadAxis(yPin);

  //Y- axis
Serial.println("*Positionner l'axe Y- vers le plafond et appuyer sur 1");    
while(pythonMessage!='1'){
    if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
}
      // Samples readings
pythonMessage ='0';
yMinus = ReadAxis(yPin);


//-------------Communication with python script ------------------------------
pythonMessage = 0;
Serial.println("Transmission des informations au script Python");
delay(1000);

Serial.println("DEBUT");
// Sending power values issued by DAC
Serial.print(xMinus);
Serial.print(";");
Serial.print(xZero);
Serial.print(";");
Serial.println(xPlus);
Serial.print(yMinus);
Serial.print(";");
Serial.print(yZero);
Serial.print(";");
Serial.println(yPlus);
Serial.print(zMinus);
Serial.print(";");
Serial.print(zZero);
Serial.print(";");
Serial.println(zPlus);
Serial.println("FIN");

Serial.println();
Serial.println();
Serial.println();
Serial.println();

delay(1000);
}

// ------------ Functions---------------------------------

//MEAN :  average of samplingsNb samples on given axisPin    
int ReadAxis(int axisPin){
  long reading = 0;
  analogRead(axisPin);
  delay(1);
  for (int i = 0; i < samplingsNb; i++)
  {
    reading += analogRead(axisPin);
  }
  return reading/samplingsNb;
}

