// DATA-ACQUISITION (3-AXIS ACCELEROMETER ADXL326)
// Author : Solène Kojtych
// solene.kojtych@polymtl.ca
// 2018

//Wiring (accelerometer pin -------- Arduino pin)
// power                       :     Vin     --------   power 3.3V 
// ground                      :     GND   --------   GND
//  voltage reference  :     3Vo     --------   AREF
// X axis                      :      Xout  --------   A0
// Y axis                      :      Yout  --------   A1
// Z axis                      :       Zout --------   A2

//--------------- PARAMETERS ----------------------
const float fs = 1000;       // Sampling frequency in Hz
const int T = 3;                  // Sampling duration in seconds
//---------------------------------------------------------------

// Constants
const int xPin = A2;
const int yPin = A1;
const int zPin = A0;
const int long serialDataRate =2000000;    
const unsigned long int samplingsNb = fs*T;  

// Variables
char pythonMessage = 0;
int xValue = 0;
int yValue = 0;
int zValue = 0;
unsigned long startTime;
unsigned long int counter = 0;
unsigned long int copyCounter =0;
boolean endMessage = true;
String stringValues;

// Librairy necessary for accurate sampling
#include <TimerOne.h>             // enables timer interrupts        

void setup(){
  analogReference(EXTERNAL);
  Serial.begin(serialDataRate); 
  Serial.println("DEBUT");

  // Data acquisition trigger
  while(pythonMessage!='1'){
  if (Serial.available() > 0){
      pythonMessage = Serial.read();
    }
  }
  startTime = micros();

  // Timer1 I initialization at frequency fs
  Timer1.initialize(fs);
  Timer1.attachInterrupt(interruption);
}

// Tasks attached to Timer 1 interruption
void interruption(void){
   Serial.print(micros()-startTime);       //write time
   xValue = analogRead(xPin);               //record values
   yValue = analogRead(yPin);
   zValue = analogRead(zPin);

   stringValues = ";";                     //assemble results
   stringValues += xValue ;
   stringValues += ";";
   stringValues += yValue ;
   stringValues += ";";
   stringValues += zValue ;
   
   Serial.println(stringValues);     //data sending
   counter = counter+1;                   // number of samples taken
}

void loop(){
  // Reading counter's value
  noInterrupts();
  copyCounter = counter;       
  interrupts();
  
  // Stops sampling procedure after samplingsNb + 1 samples 
  // to ignore first incomplete sample 
  if(copyCounter >samplingsNb && endMessage==true) {
    Timer1.detachInterrupt();         //stops timer interruption         
    endMessage=false;           
    Serial.println("FIN");            //final trigger for python script 
  } 
}
